<?php
/**
 * $Horde: wicked/lib/Page/EditPage.php,v 1.31 2008-01-23 15:32:44 jan Exp $
 *
 * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @package Wicked
 */

/**
 * Page
 */
require_once WICKED_BASE . '/lib/Page.php';

/**
 * Wicked EditPage class.
 *
 * @author  Jason M. Felice <jason.m.felice@gmail.com>
 * @package Wicked
 */
class EditPage extends Page {

    /**
     * Display modes supported by this page.
     *
     * @var array
     */
    var $supportedModes = array(
        WICKED_MODE_DISPLAY => true,
        WICKED_MODE_EDIT => true);

    /**
     * The page that we're editing.
     *
     * @var string
     */
    var $_referrer = null;

    function EditPage($referrer)
    {
        $this->_referrer = $referrer;
    }

    /**
     * Retrieve this user's permissions for the referring page.
     *
     * @return integer  The permissions bitmask.
     */
    function getPermissions()
    {
        return parent::getPermissions($this->referrer());
    }

    /**
     * Send them back whence they came if they aren't allowed to edit
     * this page.
     */
    function preDisplay()
    {
        if (!$this->allows(WICKED_MODE_EDIT)) {
            header('Location: ' . Wicked::url($this->referrer(), true));
            exit;
        }
    }

    /**
     * Render this page in Display mode.
     *
     * @return mixed  Returns true or PEAR_Error.
     */
    function display()
    {
        $page = Page::getPage($this->referrer());
        $page_text = Util::getFormData('page_text');
        if (is_null($page_text)) {
            $page_text = $page->getText();
        }

        require WICKED_TEMPLATES . '/edit/standard.inc';
        return true;
    }

    function pageName()
    {
        return 'EditPage';
    }

    function pageTitle()
    {
        return _("EditPage");
    }

    function referrer()
    {
        return $this->_referrer;
    }

    function handleAction()
    {
        global $notification, $conf;

        $page = Page::getPage($this->referrer());
        if (!$this->allows(WICKED_MODE_EDIT)) {
            $notification->push(sprintf(_("You don't have permission to edit \"%s\"."), $page->pageName()));
        } else {
            if (!empty($GLOBALS['conf']['wicked']['captcha']) &&
                !Auth::getAuth() &&
                (String::lower(Util::getFormData('wicked_captcha')) != String::lower(Wicked::getCAPTCHA()))) {
                $notification->push(_("Random string did not match."), 'horde.error');
                return;
            } 
            $text = Util::getFormData('page_text');
            $changelog = Util::getFormData('changelog');
            if ($conf['wicked']['require_change_log'] && empty($changelog)) {
                $notification->push(_("You must provide a change log."), 'horde.error');
                $notification->push('if (document.editform && document.editform.changelog) document.editform.changelog.focus();', 'javascript');
                return;
            }
            $minorchange = Util::getFormData('minor');
            if (trim($text) == trim($page->getText())) {
                $notification->push(_("No changes made"), 'horde.warning');
            } else {
                $result = $page->updateText($text, $changelog, $minorchange);
                if (is_a($result, 'PEAR_Error')) {
                    $notification->push(sprintf(_("Save Failed: %s"),
                                                $result->getMessage()), 'horde.error');
                } else {
                    $notification->push(_("Page Saved"), 'horde.success');
                }
            }
        }

        // Show the newly saved page.
        header('Location: ' . Wicked::url($this->referrer(), true));
        exit;
    }

}
